#include "stdafx.h"
#include "RedAverage.h"
#include <string.h>



void FillMem(unsigned char *ptr, unsigned char value, int width, int height, int pitch)
{
	for (int y=0; y<height; y++)
	{
		memset(ptr, value, width);
		ptr += pitch;
	}
}



void CommonFilter::CheckFrameSize(IScriptEnvironment *env)
{
	int src_width =vi.width;
	int src_height =( lsb==1 ? vi.height*2 : (lsb==2 ? vi.height/2 : vi.height ));
	if ((lsb & 1) && ((src_height % 4) !=0))
		env->ThrowError("%s : 16bit clip must have height mod4.", filterName);

	VideoInfo vi1;
	for (unsigned i=1; i < clips.size(); ++i) {
		vi1 = clips[i]->GetVideoInfo ();
		if (vi1.width  != src_width) 
			env->ThrowError("%s : Make up your mind, resize the input clips and try again.", filterName);
		if (vi1.height != src_height) 
			env->ThrowError("%s : Make up your mind, resize the input clips and try again.", filterName);
	}
}



void CommonFilter::CheckSSE(int _sse, IScriptEnvironment *env)
{
	optimize=0;
	if (((env->GetCPUFlags() & CPUF_INTEGER_SSE) !=0) && ((_sse &  1)!=0)) optimize+=1;
	if (((env->GetCPUFlags() & CPUF_SSE)         !=0) && ((_sse &  2)!=0)) optimize+=2;
	if (((env->GetCPUFlags() & CPUF_SSE2)        !=0) && ((_sse &  4)!=0)) optimize+=4;
	if (((env->GetCPUFlags() & CPUF_SSSE3)       !=0) && ((_sse &  8)!=0)) optimize+=8;
	if (((env->GetCPUFlags() & CPUF_SSE4_1)      !=0) && ((_sse & 16)!=0)) optimize+=16;
	if (clips.size() > MaxLen) optimize=0;
}



CommonFilter::CommonFilter(PClip _child, vector<PClip> _clips, double _bias, int _y, int _u, int _v, int _sse, bool _lsb_in, bool _lsb_out, IScriptEnvironment *env, const char *name)
	: GenericVideoFilter(_child), clips(_clips)
{
	if(clips.size()==0)
		env->ThrowError("%s : Boring....gimme some input clips.", name);

	maxbitin=(lsb_in ? 65536 : 256);
	maxbitout=(lsb_out ? 65536 : 256);

	if (_y-COPY_SOURCE_3>int(clips.size())-3)
		Y=OPERATION;
	else if (_y<=-maxbitout)
		Y=1-maxbitout;
	else
		Y=_y;
	if (_u-COPY_SOURCE_3>int(clips.size())-3)
		U=OPERATION;
	else if (_u<=-maxbitout)
		U=1-maxbitout;
	else
		U=_u;
	if (_v-COPY_SOURCE_3>int(clips.size())-3)
		V=OPERATION;
	else if (_v<=-maxbitout)
		V=1-maxbitout;
	else
		V=_v;

	bias=_bias;
	lsb_in =_lsb_in;	
	lsb_out=_lsb_out;
	lsb=(lsb_in ? 1 : 0)+(lsb_out ? 2 : 0);
	length=clips.size();
	filterName=env->SaveString(name);

	VideoInfo vi1 = clips[0]->GetVideoInfo ();
	vi.width  =  vi1.width;
	vi.height =( lsb==1 ? vi1.height/2 : (lsb==2 ? vi1.height*2 : vi1.height ));

	CheckSSE(_sse, env);
}



void CommonFilter::DontProcessPlane(int plane, int yuv, IScriptEnvironment* env)
{
	int inPlace=-1; //meanwhile
	if (yuv<=0)
		FillMem(result->GetWritePtr(plane), yuv, result->GetRowSize(plane), result->GetHeight(plane), result->GetPitch(plane));
	else if (yuv>1) {
		yuv=YUVidx(yuv);
		if (inPlace!=yuv)
			env->BitBlt(result->GetWritePtr(plane), result->GetPitch(plane), 
						frames[yuv]->GetReadPtr(plane), frames[yuv]->GetPitch(plane), 
						min(frames[yuv]->GetRowSize(plane),result->GetRowSize(plane)), 
						min(frames[yuv]->GetHeight(plane), result->GetHeight(plane)));
	}
}



void CommonFilter::ProcessFrame(IScriptEnvironment* env, VideoInfo vi)
{
	ChooseDestination(env);

	if (Y==OPERATION) 
		ProcessPlane(PLANAR_Y, env);
	else
		DontProcessPlane(PLANAR_Y, Y, env);
	if (U==OPERATION) 
		ProcessPlane(PLANAR_U, env);
	else
		DontProcessPlane(PLANAR_U, U,env);
	if (V==OPERATION) 
		ProcessPlane(PLANAR_V, env);
	else
		DontProcessPlane(PLANAR_V, V,env);
}



void CommonFilter::ChooseDestination(IScriptEnvironment *env)
{
	result = env->NewVideoFrame(vi,PitchAlign);
	//makewritable bad
	//if (frames[2]->IsWritable()) 
	//	result=frames[2];
	//else
	//	result = env->NewVideoFrame(vi,PitchAlign);
}


