#include "stdafx.h"
#include "RedAverage.h"
//#include <valarray>
#include <math.h>

RAverageM::RAverageM(PClip _child, vector<PClip> _clips, double _bias, int _y, int _u, int _v, int _sse, bool _lsb_in, bool _lsb_out, IScriptEnvironment *env)
	: GenericVideoFilter(_child), clips(_clips)
{
	if(clips.size()==0)
		env->ThrowError("RAverageM: Boring....gimme some input clips.");
	Y=_y;	U=_u;	V=_v;
	bias=_bias;
	sse=_sse;
	lsb_in =_lsb_in;	
	lsb_out=_lsb_out;
	lsb=(lsb_in ? 1 : 0)+(lsb_out ? 2 : 0);
	maxbit=(lsb_out ? 65536 : 256);
	length=clips.size()/2;

	if(bias>maxbit || bias<-length*maxbit*2)
		env->ThrowError("RAverageM: Look at your bias buddy...even PC scale has its limits.");

	optimize=0;
	//if (((env->GetCPUFlags() & CPUF_INTEGER_SSE) !=0) && ((sse &  1)!=0)) optimize+=1;
	//if (((env->GetCPUFlags() & CPUF_SSE)         !=0) && ((sse &  2)!=0)) optimize+=2;
	if (((env->GetCPUFlags() & CPUF_SSE2)        !=0) && ((sse &  4)!=0)) optimize+=4;
	if (((env->GetCPUFlags() & CPUF_SSSE3)       !=0) && ((sse &  8)!=0)) optimize+=8;
	if (((env->GetCPUFlags() & CPUF_SSE4_1)      !=0) && ((sse & 16)!=0)) optimize+=16;
	if (clips.size() > MaxLen) optimize=0;

	VideoInfo vi1 = clips[0]->GetVideoInfo ();
	int src_width =vi1.width;
	int src_height=vi1.height;
	if ((lsb & 1) && ((src_height % 4) !=0))
		env->ThrowError("RAverageM: 16bit clip must have height mod4.");

	for (unsigned i=1; i < _clips.size(); ++i) {
		vi1 = clips[i]->GetVideoInfo ();
		if (vi1.width  != src_width) 
			env->ThrowError("RAverageM: Make up your mind, resize the input clips and try again.");
		if (vi1.height != src_height) 
			env->ThrowError("RAverageM: Make up your mind, resize the input clips and try again.");
	}
	vi.width  =  src_width;
	vi.height =( lsb==1 ? src_height/2 : (lsb==2 ? src_height*2 : src_height ));
}
		
PVideoFrame __stdcall RAverageM::GetFrame(int n, IScriptEnvironment *env) 
{
	frames.clear();
	frames.reserve(clips.size());
	for (unsigned i=0; i < clips.size(); ++i)
		frames.push_back(clips[i]->GetFrame(n, env));
	frames.clear();
	return WeightedAverageM16(env, vi);
}

AVSValue __cdecl Create_RAverageM(AVSValue arguments, void *user_data, IScriptEnvironment *env) 
{
	AVSValue arrayy = arguments[0];
	int noarguments = arrayy.ArraySize();
	if(noarguments % 2 != 0)
		env->ThrowError("RAverageM: RAverageM requires an even number of clips.");

	vector<PClip> clips;
	for (int i=0; i < noarguments; i++) {
		if (!arrayy[i].IsClip())
            env->ThrowError("RAverageM: Argument is not a clip");
		clips.push_back(arrayy[i].AsClip());
	}

	return new RAverageM(clips[0], clips, arguments[1].AsFloat(0), arguments[2].AsInt(3), arguments[3].AsInt(3), arguments[4].AsInt(3), arguments[5].AsInt(-1), arguments[6].AsBool(false), arguments[7].AsBool(false),env);  
}





RAverageW::RAverageW(PClip _child, vector<WeightedClip> _clips, double _bias, int _y, int _u, int _v, int _sse, bool _lsb_in, bool _lsb_out, int _mode, int _errlevel, bool _silent, IScriptEnvironment *env)
	: GenericVideoFilter(_child), clips(_clips)
{
	_mode=_mode & (4+8); //only these methods are working now
	if(_clips.size()==0)
		env->ThrowError("RAverageW: Boring....gimme some input clips.");
	Y=_y;	U=_u;	V=_v;
	bias=_bias;
	sse=_sse;
	lsb_in =_lsb_in;	
	lsb_out=_lsb_out;
	lsb=(lsb_in ? 1 : 0)+(lsb_out ? 2 : 0);
	maxbit=(lsb_out ? 65536 : 256);
	maxbitsrc=(lsb_in ? 65536 : 256);

	optimize=0;
	//if (((env->GetCPUFlags() & CPUF_INTEGER_SSE) !=0) && ((sse &  1)!=0)) optimize+=1;
	//if (((env->GetCPUFlags() & CPUF_SSE)         !=0) && ((sse &  2)!=0)) optimize+=2;
	if (((env->GetCPUFlags() & CPUF_SSE2)        !=0) && ((sse &  4)!=0)) optimize+=4;
	if (((env->GetCPUFlags() & CPUF_SSSE3)       !=0) && ((sse &  8)!=0)) optimize+=8;
	if (((env->GetCPUFlags() & CPUF_SSE4_1)      !=0) && ((sse & 16)!=0)) optimize+=16;
	if (clips.size() > MaxLen) optimize=0;
	if ((_mode == 4) && !(optimize & 8))
		env->ThrowError("RAverageW: Mode 4 requires SSSE3.");
	else if ((_mode & 4) && !(optimize & 8))
		_mode= (_mode | 4) ^ 4;
	if (_lsb_in && (_mode==4))
		env->ThrowError("RAverageW: 16bits not supported in mode 4.");
	if (_lsb_in)
		_mode=(_mode | 4) ^ 4;

	method=AssessWeights16(_mode, _errlevel, _silent, env);

	if(length==0)
		env->ThrowError("RAverageW: Well, I looked at your weights, I really did... but I think it is waste of resources...sooo, I am going for a sickday. I mean, unless there is an emergency and you want me to save the world.");
	
	VideoInfo vi1 = selectedclips[0]->GetVideoInfo ();
	int src_width =vi1.width;
	int src_height=vi1.height;
	if ((lsb & 1) && ((src_height % 4) !=0))
		env->ThrowError("RAverageW: 16bit clip must have height mod4.");

	for (int i=1; i < length; ++i) {
		vi1 = selectedclips[i]->GetVideoInfo ();
		if (vi1.width  != src_width) 
			env->ThrowError("RAverageW: Make up your mind, resize the input clips and try again.");
		if (vi1.height != src_height) 
			env->ThrowError("RAverageW: Make up your mind, resize the input clips and try again.");
	}
	vi.width  =  src_width;
	vi.height =( lsb==1 ? src_height/2 : (lsb==2 ? src_height*2 : src_height ));
}



PVideoFrame __stdcall RAverageW::GetFrame(int n, IScriptEnvironment *env) 
{
	frames.clear();
	frames.reserve(length);
	for (int i=0; i < length; ++i)
		frames.push_back(selectedclips[i]->GetFrame(n, env));
	return WeightedAverageW16(env, vi);
}

//PVideoFrame __stdcall RAverageW::GetFrame(int n, IScriptEnvironment *env) 
//{
//	vector<WeightedFrame> data;
//		data.reserve(clips.size());
//	for (unsigned i=0; i < clips.size(); ++i)
//		data.push_back(WeightedFrame(clips[i].clip->GetFrame(n, env), clips[i].weight));
//
//	return WeightedAverageW(data, bias, Y, U, V, sse, env, vi);
//}
//
AVSValue __cdecl Create_RAverageW(AVSValue arguments, void *user_data, IScriptEnvironment *env) 
{
	AVSValue array = arguments[0];
	int noarguments = array.ArraySize();
	if(noarguments % 2 != 0)
		env->ThrowError("RAverageW: Averaging arguments need to be in pairs.");
	vector<WeightedClip> clips;
	for (int i=0; i < noarguments; i+=2) {
		if (!array[i].IsClip())
            env->ThrowError("RAverageW: Odd argument is not clip");
		if (!array[i+1].IsFloat())
            env->ThrowError("RAverageW: Even argument is not a number");
		clips.push_back(WeightedClip(array[i].AsClip(), array[i+1].AsFloat()));
	}
	return new RAverageW(clips[0].clip, clips, arguments[1].AsFloat(0), arguments[2].AsInt(3), arguments[3].AsInt(3), arguments[4].AsInt(3), arguments[5].AsInt(-1), arguments[6].AsBool(false), arguments[7].AsBool(false), arguments[8].AsInt(-1), arguments[9].AsInt(0), arguments[10].AsBool(false),env);  
}











PVideoFrame __stdcall RMerge::GetFrame(int n, IScriptEnvironment *env) 
{
	frames.clear();
	frames.reserve(clips.size());
	for (unsigned i=0; i < clips.size(); ++i)
		frames.push_back(clips[i]->GetFrame(n, env));
	ProcessFrame(env, vi);
	frames.clear();
	return result;
}



RMerge::RMerge(PClip _child, vector<PClip> _clips, double _bias, int _y, int _u, int _v, int _sse, bool _lsb_in, bool _lsb_out, int _mode, IScriptEnvironment *env) 
	: CommonFilter(_child, _clips, _bias, _y, _u, _v, _sse, _lsb_in, _lsb_out, env, "RMerge")//, clips(_clips)
{
	CheckFrameSize(env);
	if(!_lsb_in)
		if ((_mode!=255) && (_mode!=256))
			env->ThrowError("RMerge: Mode can be 255 or 256 in 8bit input.");
	if((optimize!=0) && (_mode==256))
		env->ThrowError("RMerge: SSE not implemented for mode 256 yet.");
	mode=_mode;
}


AVSValue __cdecl Create_RMerge(AVSValue arguments, void *user_data, IScriptEnvironment *env) 
{
	double _bias=0;
	vector<PClip> clips;
	clips.push_back(arguments[0].AsClip());
	clips.push_back(arguments[1].AsClip());
	clips.push_back(arguments[2].AsClip());
	return new RMerge(clips[0], clips, _bias, arguments[3].AsInt(3), arguments[4].AsInt(3), arguments[5].AsInt(3), arguments[6].AsInt(-1), arguments[7].AsBool(false), arguments[8].AsBool(false), arguments[9].AsInt(255),env);  
}









extern "C" __declspec(dllexport) const char *__stdcall AvisynthPluginInit2(IScriptEnvironment *env) 
{
	env->AddFunction("RAverageM", ".*[bias]f[y]i[u]i[v]i[sse]i[lsb_in]b[lsb_out]b", Create_RAverageM, 0);
	env->AddFunction("RAverageW", ".*[bias]f[y]i[u]i[v]i[sse]i[lsb_in]b[lsb_out]b[mode]i[errlevel]i[silent]b", Create_RAverageW, 0);
	env->AddFunction("RMerge", "ccc[y]i[u]i[v]i[sse]i[lsb_in]b[lsb_out]b[mode]i", Create_RMerge, 0);
//	env->AddFunction("RMerge", "ccc[bias]f[y]i[u]i[v]i[sse]i[lsb_in]b[lsb_out]b[mode]i", Create_RMerge, 0);
	//RTotal sum cpl rec
	return "RedAverage plugin";
}
